@@ -17,18 +17,20 @@
 #
 """
 usage: cb-ctl -h | --help
-       cb-ctl --build-local --dist=<name>
+       cb-ctl --build-package-local --dist=<name>
            [--clean]
-       cb-ctl --build=<package> --project-path=<path> --arch=<name> --dist=<name> --runner-group=<name>
+       cb-ctl --build-package=<package> --project-path=<path> --arch=<name> --dist=<name> --runner-group=<name>
            [--clean]
-       cb-ctl --build-dependencies=<package> --project-path=<path> --arch=<name> --dist=<name>
+       cb-ctl --build-image-local --selection=<name>
+       cb-ctl --build-image=<image> --project-path=<path> --arch=<name> --runner-group=<name> --selection=<name>
+       cb-ctl --build-dependencies=<package│image> --project-path=<path> --arch=<name> (--dist=<name>|--selection=<name>)
            [--timeout=<time_sec>]
-       cb-ctl --build-dependencies-local --dist=<name>
-       cb-ctl --build-log=<package> --project-path=<path> --arch=<name> --dist=<name>
+       cb-ctl --build-dependencies-local (--dist=<name>|--selection=<name>)
+       cb-ctl --build-log=<package│image> --project-path=<path> --arch=<name> (--dist=<name>|--selection=<name>)
            [--timeout=<time_sec>]
-       cb-ctl --build-info=<package> --project-path=<path> --arch=<name> --dist=<name>
+       cb-ctl --build-info=<package│image> --project-path=<path> --arch=<name> (--dist=<name>|--selection=<name>)
            [--timeout=<time_sec>]
-       cb-ctl --get-binaries=<package> --project-path=<path> --arch=<name> --dist=<name> --target-dir=<dir>
+       cb-ctl --get-binaries=<package│image> --project-path=<path> --arch=<name> --target-dir=<dir> (--dist=<name>|--selection=<name>)
            [--timeout=<time_sec>]
        cb-ctl --watch
            [--filter-request-id=<uuid>]
@@ -36,24 +38,33 @@ usage: cb-ctl -h | --help
            [--timeout=<time_sec>]
 
 options:
-    --build=<package>
+    --build-package=<package>
         Create a request to build the given package.
         The provided argument is appended to the
         project-path and forms the directory path
         to the package in the git repository
 
         projects/
-        └── <project-path>/
+          └── <project-path>/
                     └── <package>/...
 
         Please note, the root directory is by convention
         a fixed name set to 'projects'
 
-    --build-local
+    --build-package-local
         Build package from local checkout. The package
         sources will be looked up from the current working
         directory
 
+    --build-image-local
+        Build image from local checkout. The image sources
+        will be looked up from the current working directory
+
+    --build-image=<image>
+        Create a request to build the given image.
+        The provided image argument is used in the same
+        way as the package argument from --build-package
+
     --project-path=<path>
         Project path that points to the package in the git.
         See the above structure example
@@ -62,24 +73,27 @@ options:
         Target architecture name
 
     --dist=<name>
-        Target distribution name
+        Target distribution name for package builds
+
+    --selection=<name>
+        Image selection name for image builds
 
     --runner-group=<name>
         Send build request to specified runner group
 
-    --build-dependencies=<package>
+    --build-dependencies=<package|image>
         Provide latest build root dependency information
 
     --build-dependencies-local
         Calculate build dependencies now and on the local system
 
-    --build-log=<package>
+    --build-log=<package│image>
         Provide latest raw package build log
 
-    --build-info=<package>
+    --build-info=<package│image>
         Provide latest build result and status information
 
-    --get-binaries=<package>
+    --get-binaries=<package│image>
         Download latest binary packages
 
     --target-dir=<dir>
@@ -99,10 +113,11 @@ options:
         * cb-run
         * cb-prepare
         * cb-scheduler
+        * cb-image
 
     --timeout=<time_sec>
         Wait time_sec seconds of inactivity on the message
-        broker before return. Default: 30sec
+        broker before return. Default: 10sec
 
     --clean
         Delete package buildroot if present on the runner
@@ -124,19 +139,23 @@ from typing import (
 from cloud_builder.version import __version__
 from cloud_builder.defaults import Defaults
 from cloud_builder.broker import CBMessageBroker
-from cloud_builder.package_request.package_request import CBPackageRequest
-from cloud_builder.package_metadata.package_metadata import CBPackageMetaData
+from cloud_builder.build_request.build_request import CBBuildRequest
+from cloud_builder.project_metadata.project_metadata import CBProjectMetaData
 from cloud_builder.info_request.info_request import CBInfoRequest
 from cloud_builder.utils.display import CBDisplay
 from cloud_builder.config.cbctl_schema import cbctl_config_schema
-from cloud_builder.cb_scheduler import create_run_script
+from cloud_builder.cb_scheduler import (
+    create_package_run_script,
+    create_image_run_script,
+    is_request_valid
+)
 from cloud_builder.cb_prepare import resolve_build_dependencies
 
 from cloud_builder.exceptions import (
     exception_handler,
     CBConfigFileNotFoundError,
     CBConfigFileValidationError,
-    CBPackageMetadataError
+    CBProjectMetadataError
 )
 
 from kiwi.command import Command
@@ -160,21 +179,33 @@ def main() -> None:
         version='CB (ctl) version ' + __version__,
         options_first=True
     )
-    if args['--build']:
+    default_timeout = 10
+    if args['--build-package']:
         build_package(
             get_broker(),
-            args['--build'],
+            args['--build-package'],
             args['--project-path'],
             args['--arch'],
             args['--dist'],
             args['--runner-group'],
             bool(args['--clean'])
         )
-    elif args['--build-local']:
+    elif args['--build-package-local']:
         build_package_local(
             args['--dist'],
             bool(args['--clean'])
         )
+    elif args['--build-image-local']:
+        build_image_local(args['--selection'])
+    elif args['--build-image']:
+        build_image(
+            get_broker(),
+            args['--build-image'],
+            args['--project-path'],
+            args['--arch'],
+            args['--selection'],
+            args['--runner-group']
+        )
     elif args['--build-dependencies']:
         get_build_dependencies(
             get_broker(),
@@ -182,7 +213,7 @@ def main() -> None:
             args['--project-path'],
             args['--arch'],
             args['--dist'],
-            int(args['--timeout'] or 30),
+            int(args['--timeout'] or default_timeout),
             get_config()
         )
     elif args['--build-dependencies-local']:
@@ -196,7 +227,7 @@ def main() -> None:
             args['--project-path'],
             args['--arch'],
             args['--dist'],
-            int(args['--timeout'] or 30),
+            int(args['--timeout'] or default_timeout),
             get_config()
         )
     elif args['--build-info']:
@@ -206,7 +237,7 @@ def main() -> None:
             args['--project-path'],
             args['--arch'],
             args['--dist'],
-            int(args['--timeout'] or 30)
+            int(args['--timeout'] or default_timeout)
         )
     elif args['--get-binaries']:
         fetch_binaries(
@@ -215,12 +246,12 @@ def main() -> None:
             args['--project-path'],
             args['--arch'],
             args['--dist'],
-            int(args['--timeout'] or 30),
+            int(args['--timeout'] or default_timeout),
             args['--target-dir'],
             get_config()
         )
     elif args['--watch']:
-        timeout = int(args['--timeout'] or 30)
+        timeout = int(args['--timeout'] or default_timeout)
         if args['--filter-request-id']:
             _response_reader(
                 get_broker(), timeout, watch_filter_request_id(
@@ -265,24 +296,53 @@ def build_package(
     arch: str, dist: str, runner_group: str, clean_buildroot: bool
 ) -> None:
     status_flags = Defaults.get_status_flags()
-    package_request = CBPackageRequest()
+    package_request = CBBuildRequest()
     package_request.set_package_build_request(
-        _get_package_path(project_path, package),
+        _get_target_path(project_path, package),
         arch, dist, runner_group,
         status_flags.package_rebuild_clean if clean_buildroot else
         status_flags.package_rebuild
     )
-    broker.send_package_request(package_request)
+    broker.send_build_request(package_request)
     CBDisplay.print_json(package_request.get_data())
     broker.close()
 
 
+def build_image_local(selection: str) -> None:
+    Privileges.check_for_root_permissions()
+
+    status_flags = Defaults.get_status_flags()
+    image_source_path = os.getcwd()
+    image_request = CBBuildRequest()
+    image_request.set_image_build_request(
+        image=image_source_path,
+        arch=platform.machine(),
+        selection=selection,
+        runner_group='local',
+        action=status_flags.image_local
+    )
+
+    project_config = _check_project_config_from_working_directory(
+        image_request
+    )
+
+    image_build_run = [
+        'bash', create_image_run_script(
+            image_request.get_data(), project_config, local_build=True
+        )
+    ]
+    exit_code = os.WEXITSTATUS(
+        os.system(' '.join(image_build_run))
+    )
+    sys.exit(exit_code)
+
+
 def build_package_local(dist: str, clean_buildroot: bool) -> None:
     Privileges.check_for_root_permissions()
 
     status_flags = Defaults.get_status_flags()
     package_source_path = os.getcwd()
-    package_request = CBPackageRequest()
+    package_request = CBBuildRequest()
     package_request.set_package_build_request(
         package=package_source_path,
         arch=platform.machine(),
@@ -291,10 +351,10 @@ def build_package_local(dist: str, clean_buildroot: bool) -> None:
         action=status_flags.package_local
     )
 
-    _check_package_config_from_working_directory()
+    _check_project_config_from_working_directory(package_request)
 
     package_build_run = [
-        'bash', create_run_script(
+        'bash', create_package_run_script(
             package_request.get_data(), clean_buildroot,
             local_build=True
         )
@@ -305,12 +365,28 @@ def build_package_local(dist: str, clean_buildroot: bool) -> None:
     sys.exit(exit_code)
 
 
+def build_image(
+    broker: Any, image: str, project_path: str,
+    arch: str, selection: str, runner_group: str
+) -> None:
+    status_flags = Defaults.get_status_flags()
+    image_request = CBBuildRequest()
+    image_request.set_image_build_request(
+        _get_target_path(project_path, image),
+        arch, selection, runner_group,
+        status_flags.image_rebuild
+    )
+    broker.send_build_request(image_request)
+    CBDisplay.print_json(image_request.get_data())
+    broker.close()
+
+
 def get_build_dependencies(
-    broker: Any, package: str, project_path: str, arch: str, dist: str,
+    broker: Any, target: str, project_path: str, arch: str, dist: str,
     timeout_sec: int, config: Dict
 ) -> None:
     solver_data = _get_info_response_file(
-        broker, package, project_path, arch, dist,
+        broker, target, project_path, arch, dist,
         timeout_sec, config, 'solver_file'
     )
     if solver_data:
@@ -320,9 +396,12 @@ def get_build_dependencies(
 def get_build_dependencies_local(dist: str) -> None:
     Privileges.check_for_root_permissions()
 
-    package_source_path = _check_package_config_from_working_directory()
+    target_source_path = os.getcwd()
+
+    _check_project_config_from_working_directory()
+
     solver_result = resolve_build_dependencies(
-        package_source_path, [f'{dist}.{platform.machine()}']
+        target_source_path, [f'{dist}.{platform.machine()}']
     )
     if solver_result['solver_data']:
         CBDisplay.print_json(
@@ -335,11 +414,11 @@ def get_build_dependencies_local(dist: str) -> None:
 
 
 def get_build_log(
-    broker: Any, package: str, project_path: str, arch: str, dist: str,
+    broker: Any, target: str, project_path: str, arch: str, dist: str,
     timeout_sec: int, config: Dict
 ) -> None:
     build_log_data = _get_info_response_file(
-        broker, package, project_path, arch, dist,
+        broker, target, project_path, arch, dist,
         timeout_sec, config, 'log_file'
     )
     if build_log_data:
@@ -347,20 +426,20 @@ def get_build_log(
 
 
 def get_build_info(
-    broker: Any, package: str, project_path: str, arch: str, dist: str,
+    broker: Any, target: str, project_path: str, arch: str, dist: str,
     timeout_sec: int
 ) -> None:
     CBDisplay.print_json(
-        get_info(broker, package, project_path, arch, dist, timeout_sec)
+        get_info(broker, target, project_path, arch, dist, timeout_sec)
     )
 
 
 def fetch_binaries(
-    broker: Any, package: str, project_path: str, arch: str, dist: str,
+    broker: Any, target: str, project_path: str, arch: str, dist: str,
     timeout_sec: int, target_dir, config: Dict
 ) -> None:
     info_response = get_info(
-        broker, package, project_path, arch, dist, timeout_sec
+        broker, target, project_path, arch, dist, timeout_sec
     )
     if info_response:
         runner_ip = info_response['source_ip']
@@ -395,7 +474,8 @@ def watch_filter_service_name(service_name: str) -> Callable:
             'cb-info': 'CBInfo',
             'cb-run': 'CBRun',
             'cb-prepare': 'CBPrepare',
-            'cb-scheduler': 'CBScheduler'
+            'cb-scheduler': 'CBScheduler',
+            'cb-image': 'CBImage'
         }
         if response['identity'].startswith(service_id[service_name]):
             CBDisplay.print_json(response)
@@ -429,21 +509,21 @@ def watch_filter_none() -> Callable:
 
 
 def get_info(
-    broker: Any, package: str, project_path: str, arch: str, dist: str,
+    broker: Any, target: str, project_path: str, arch: str, dist: str,
     timeout_sec: int
 ) -> Dict:
     request_id = _send_info_request(
-        broker, _get_package_path(project_path, package), arch, dist
+        broker, _get_target_path(project_path, target), arch, dist
     )
     return _info_reader(broker, request_id, timeout_sec)
 
 
 def _get_info_response_file(
-    broker: Any, package: str, project_path: str, arch: str, dist: str,
+    broker: Any, target: str, project_path: str, arch: str, dist: str,
     timeout_sec: int, config: Dict, response_file_name
 ) -> str:
     info_response = get_info(
-        broker, package, project_path, arch, dist, timeout_sec
+        broker, target, project_path, arch, dist, timeout_sec
     )
     if info_response:
         response_file = info_response[response_file_name]
@@ -482,7 +562,7 @@ def _response_reader(
                 group=f'cb-ctl:{os.getpid()}',
                 timeout_ms=timeout_sec * 1000
             ):
-                response = broker.validate_package_response(
+                response = broker.validate_build_response(
                     message.value
                 )
                 if response:
@@ -495,10 +575,10 @@ def _response_reader(
 
 
 def _send_info_request(
-    broker: Any, package_path: str, arch: str, dist: str
+    broker: Any, target_path: str, arch: str, dist: str
 ) -> str:
     info_request = CBInfoRequest()
-    info_request.set_package_info_request(package_path, arch, dist)
+    info_request.set_info_request(target_path, arch, dist)
     broker.send_info_request(info_request)
     broker.close()
     return info_request.get_data()['request_id']
@@ -507,7 +587,7 @@ def _send_info_request(
 def _info_reader(broker: Any, request_id: str, timeout_sec: int) -> Dict:
     """
     Read from the cloud builder info response queue.
-    In case multiple info services responds to the package
+    In case multiple info services responds to the package/image
     only the record of the latest timestamp will be
     used
 
@@ -562,19 +642,30 @@ def _get_datetime_from_utc_timestamp(timestamp: str) -> datetime:
     return datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S.%f")
 
 
-def _get_package_path(project_path: str, package_name: str) -> str:
+def _get_target_path(project_path: str, target_name: str) -> str:
     return os.path.join(
-        'projects', project_path, package_name,
+        'projects', project_path, target_name,
     )
 
 
-def _check_package_config_from_working_directory() -> str:
-    package_source_path = os.getcwd()
-    package_config = CBPackageMetaData.get_package_config(
-        package_source_path
-    )
-    if not package_config:
-        raise CBPackageMetadataError(
-            f'No package metadata found in {package_source_path}'
+def _check_project_config_from_working_directory(
+    request: CBBuildRequest = None
+) -> Dict:
+    target_source_path = os.getcwd()
+    if request:
+        validated_request = is_request_valid(
+            target_source_path, request.get_data()
         )
-    return package_source_path
+        if not validated_request.is_valid \
+           and validated_request.response:
+            CBDisplay.print_json(validated_request.response.get_data())
+        project_config = validated_request.project_config
+    else:
+        project_config = CBProjectMetaData.get_project_config(
+            target_source_path
+        )
+    if not project_config:
+        raise CBProjectMetadataError(
+            f'No or invalid package/image metadata in {target_source_path}'
+        )
+    return project_config
